%% Using the Worstcase Solver - Demo 1
% The |worstcase| solver is used to find the induced L2-to-L2 gain of a
% four-state nonlinear system.
%
% <html>
% Timothy J. Wheeler<br>
% Dept. of Mechanical Engineering<br>
% University of California, Berkeley
% </html>

%% Introduction.
% Consider a dynamic system of the form
%
% $$\dot{x} = f(x,u)$$
%
% $$y = g(x,u),$$
%
% where |x(0)=0|.  Given positive scalars |B| and |T|, the goal is to maximize
% 
% $$\|y\|_{2,T} := \int_0^T \|y(t)\|_2 \, dt$$
%
% subject to the constraint
% 
% $$\|u\|_{2,T} := \int_0^T \|u(t)\|_2 \, dt \le B.$$
%
% Note: we only consider inputs and outputs defined on the interval |[0,T]|.


pvar x y d
th = 0;
f = [-x-x*y+th*x+d; x^2];
g = x*y;
states = [x;y];
inputs = [d];
% Then, a |polysys| object is created from the polynomials |f| and |g|.  
sys = polysys(f,g,states,inputs);

%% Optimization parameters.
% Use the following values for the optimization parameters (defined above):
T = 10;
B = 20;

%%
% The time vector t specifies the time window (T = t(end)) and the points
% at which the system trajectory is computed.
t = linspace(0,T,100)';


%% Set options for worstcase solver.
% Create a |wcoptions| object that contains the default options.
opt = wcoptions();
%%
% Specify the maximum number of iterations and which ODE solver to use.
opt.MaxIter = 50;
opt.ODESolver = 'ode45';
%%
% Tell the solver to display a text summary of each iteration.
opt.PlotProgress = 'none';
%%
% Specify the optimization objective, and the bound on the input.
opt.Objective = 'L2';
opt.InputL2Norm = B;

%% Find worstcase input.
[tOut,x,y,u,eNorm] = worstcase(sys,t,opt);

%% Simulate with worstcase input.
% We can only compute the worstcase input over a finite interval of time
% [0,T].  However, any response of the system that occurs after the input
% is "shut off" (i.e., u(t) = 0 for t > T) should contribute to our
% objective.  Hence, we compute a more accurate value of the objective by
% continuing the simulation from the end of the previous trajectory with no
% input:
[te,xe,ye] = sim(sys,tOut,x(end,:)');
td = [tOut;tOut(2:end)+max(tOut)];
yd = [y;ye(2:end)];
%%
% The objective value over [0,T] is
eNorm
%%
% The objective value over [0,2T] is
eNormd = get2norm(yd,td)

%% Display results.

fprintf( 'The L2-to-L2 gain is %f\n', eNormd/B );

figure;
plot(tOut,u)
xlabel('Time, t')
ylabel('Input, u(t)')
title('Worst case input.')

figure;
plot(td,yd)
xlabel('Time, t')
ylabel('Output, y(t)')
title('Worst case output over extended time interval.')

